      SUBROUTINE PCPBSV( UPLO, N, BW, NRHS, A, JA, DESCA, B, IB, DESCB,
     $                   WORK, LWORK, INFO )
*
*  -- ScaLAPACK routine (version 1.2 ALPHA) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     May 10, 1996
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            BW, IB, INFO, JA, LWORK, N, NRHS
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * ), DESCB( * )
      COMPLEX            A( * ), B( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PCPBSV solves a system of linear equations
*
*                      sub( A ) * X = sub( B )
*
*  Cholesky factorization of a symmetric positive definite system
*  where sub(A) is the matrix that will produce the Cholesky factors
*  to be stored in A(1:N,JA:JA+N-1). sub( A ) is an N-by-N banded
*  symmetric positive definite distributed matrix and its Cholesky
*  factor is also N-by-N and banded.
*  Depending on the value of UPLO, A stores either U or L in the equn
*  sub( A ) = U**T*U or L*L**T as computed by PCPBTRF.
*
*  sub( B ) denotes the distributed matrix B(IB:IB+N-1,1:NRHS).
*
*  See PCPBTRF and PCPBTRS.
*
*  =====================================================================
*
*  Arguments
*  =========
*
*  UPLO    (global input) CHARACTER
*          = 'U':  Upper triangle of sub( A ) is stored;
*          = 'L':  Lower triangle of sub( A ) is stored.
*
*  N       (global input) INTEGER
*          The number of rows and columns to be operated on, i.e. the
*          order of the distributed submatrix sub( A ). N >= 0.
*
*  BW      Number of subdiagonals
*
*
*  NRHS    (global input) INTEGER
*          The number of right hand sides, i.e., the number of columns
*          of the distributed submatrix sub( B ).  NRHS >= 0.
*
*  A       (local input/local output) COMPLEX pointer into the
*          local memory to an array of dimension with first dimension
*          LLD_A.
*          On entry, this array contains the local pieces of the
*          This local portion is stored in the packed banded format
*            used in LAPACK.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension >=7
*          if 1D type, >=9 if 2D type.
*          The array descriptor for the distributed matrix A.
*
*  B       (local input/local output) DOUBLE PRECISION pointer into the
*          local memory to an array of local leading dimension lld_b.
*          On entry, this array contains the
*          the local pieces of the right hand sides sub( B ).
*          On exit, this array contains the local pieces of the solution
*          distributed matrix X.
*
*  IB      (global input) INTEGER
*          B's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  DESCB   (global and local input) INTEGER array of dimension >=7
*          if 1D type, >=9 if 2D type.
*          The array descriptor for the distributed matrix B.
*
*  INFO    (global output) INTEGER
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*          > 0:  If INFO = K, the leading minor of order K,
*                A(IA:IA+K-1,JA:JA+K-1) is not positive definite, and
*                the factorization could not be completed.
*
*  =====================================================================
*
*
*  Restrictions
*  ============
*
*  The following are restrictions on the input parameters. Many of these
*    are temporary and will be removed in future releases, while others
*    may reflect fundamental technical limitations.
*
*    Non-cyclic restriction: VERY IMPORTANT!
*      P*NB<= JA+N.
*      The mapping for matrices must be blocked, reflecting the nature
*      of the divide and conquer algorithm as a task-parallel algorithm.
*
*    Blocksize cannot be too small:
*      NB >= 2*BW
*      The bulk of parallel computation is done on the matrix of size
*      NB-BW on each processor. If this is too small, divide and conquer
*      is a poor choice of algorithm.
*
*    Submatrix reference:
*      JA = IB
*      Alignment restriction that prevents unnecessary communication.
*
*
*  =====================================================================
*
*
*  Notes
*  =====
*
*  If the factorization routine and the solve routine are to be called
*    separately (to solve various sets of righthand sides using the same
*    coefficient matrix), the auxiliary space AF *must not be altered*
*    between calls to the factorization routine and the solve routine.
*
*  The best algorithm for solving banded and tridiagonal linear systems
*    depends on a variety of parameters, especially the bandwidth.
*    Currently, only algorithms designed for the case N/P >> bw are
*    implemented. These go by many names, including Divide and Conquer,
*    Partitioning, domain decomposition-type, etc.
*
*  Algorithm description: Divide and Conquer
*
*    The Divide and Conqer algorithm assumes the matrix is narrowly
*      banded compared with the number of equations. In this situation,
*      it is best to store the matrix one-dimensionally.
*      The basic algorithm divides the banded matrix up into a small
*      number of pieces (usually P with one stored on each processor)
*      and then proceeds in 2 phases for the factorization or 3 for the
*      solution of a linear system.
*      1) Local Phase:
*         The individual pieces are factored independently and in
*         parallel. These factors are applied to the matrix creating
*         fillin. For a linear system, a local front solve is
*         performed in this stage.
*      2) Reduced System Phase:
*         A small ( BW* (P-1) ) matrix/system is formed representing the
*         interaction of the larger blocks. It can be factored in
*         several ways. Currently a parallel Block Cyclic Reduction
*         algorithm is used. For a linear system, a parallel front solve
*         followed by an analagous backsolve, both using the structure
*         of the factored matrix, are performed.
*      3) Backsubsitution Phase:
*         For a linear system, a local backsubstitution is performed on
*         each processor in parallel.
*
*  Descriptors now have *types* and differ from ScaLAPACK 1.0.
*
*  Banded codes can use either the standard two-dimensional descriptors
*    or new one-dimensional descriptors, though the processor grid in
*    both cases *must be one-dimensional*. We describe both types below.
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN      EXPLANATION
*  --------------- -------------- --------------------------------------
*  DT_A   (global) DESCA( DT_ )   The descriptor type.
*  CTXT_A (global) DESCA( CTXT_ ) The BLACS context handle, indicating
*                                 the BLACS process grid A is distribu-
*                                 ted over. The context itself is glo-
*                                 bal, but the handle (the integer
*                                 value) may vary.
*  M_A    (global) DESCA( M_ )    The number of rows in the distributed
*                                 matrix A.
*  N_A    (global) DESCA( N_ )    The number of columns in the distri-
*                                 buted matrix A.
*  MB_A   (global) DESCA( MB_ )   The blocking factor used to distribute
*                                 the rows of A.
*  NB_A   (global) DESCA( NB_ )   The blocking factor used to distribute
*                                 the columns of A.
*  RSRC_A (global) DESCA( RSRC_ ) The process row over which the first
*                                 row of the matrix A is distributed.
*  CSRC_A (global) DESCA( CSRC_ ) The process column over which the
*                                 first column of A is distributed.
*  LLD_A  (local)  DESCA( LLD_ )  The leading dimension of the local
*                                 array storing the local blocks of the
*                                 distributed matrix A.
*                                 LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*
*  One-dimensional descriptors:
*
*  One-dimensional descriptors are a new addition to ScaLAPACK since
*    version 1.0. They simplify and shorten the descriptor for 1D
*    matrices. Such a descriptor is associated with each one-
*    dimensionally distributed matrix in ScaLAPACK. This descriptor
*    allows ScaLAPACK programs to be addressed in global indices by
*    providing the means to translate between global indices and local
*    indices in a way that insulates the user from such details.
*
*  Since ScaLAPACK supports two-dimensional matrices as the fundamental
*    object, we allow for matrices to be distributed either over the
*    first dimension (as if the grid were P-by-1) or the second
*    dimension (as if the grid were 1-by-P). This choice is made in the
*    first entry of the descriptor as described below.
*    IMPORTANT NOTE: the actual BLACS grid represented by the
*    CTXT entry in the descriptor may be *either*  P-by-1 or 1-by-P
*    for either one-dimensional descriptor type. The ScaLAPACK
*    routines will interpret these properly depending on the TYPE.
*    ScaLAPACK routines *do not support intercontext operations* so that
*    the grid passed to a single ScaLAPACK routine *must be the same*
*    for all matrix descriptors passed to that routine.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  TYPE_A (global) DESCA( 1 ) The orientation of the one-dimensional
*                             grid.
*                             TYPE_A = 502: P-by-1 grid.
*                             TYPE_A = 501: 1-by-P grid.
*  CTXT_A (global) DESCA( 2 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the
*                             handle (the integer value) may vary.
*  N_A    (global) DESCA( 3 ) The size of the matrix dimension being
*                             distributed.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the distributed dimension of the matrix.
*  SRC_A  (global) DESCA( 5 ) The process row or column over which the
*                             1st element of the matrix is distributed.
*  LLD_A  (local)  DESCA( 6 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. Minimum value of LLD_A
*                             depends on TYPE_A.
*  Reserved        DESCA( 7 ) Reserved for future use.
*
*
*  =====================================================================
*
*  Implemented by: Andrew J. Cleary, May 10, 1996
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            DESCMULT
      PARAMETER          ( DESCMULT = 100 )
      INTEGER            BLOCK_CYCLIC_2D, CSRC_, CTXT_, DLEN_, DT_,
     $                   LLD_, MB_, M_, NB_, N_, RSRC_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $                     CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                     RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
      INTEGER            ICTXT, NB
*     ..
*     .. External Subroutines ..
      EXTERNAL           PCPBTRF, PCPBTRS, PXERBLA
*     ..
*     .. Executable Statements ..
*
*     Begin main code
*
      INFO = 0
*
*     Get block size to calculate workspace requirements
*
      IF( DESCA( 1 ).EQ.BLOCK_CYCLIC_2D ) THEN
         NB = DESCA( NB_ )
         ICTXT = DESCA( CTXT_ )
      ELSE IF( DESCA( 1 ).EQ.501 ) THEN
         NB = DESCA( 4 )
         ICTXT = DESCA( 2 )
      ELSE
         ICTXT = DESCA( 2 )
         INFO = -( 7*100+2 )
         CALL PXERBLA( ICTXT, 'PCPBSV', -INFO )
         RETURN
      END IF
*
*     Factor the matrix
*
      CALL PCPBTRF( UPLO, N, BW, A, JA, DESCA, WORK, LWORK-BW*BW,
     $              WORK( 1+( NB+2*BW )*BW ), BW*BW, INFO )
*
*     Check info for error conditions
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICTXT, 'PCPBSV', -INFO )
         RETURN
      END IF
*
*     Solve the system using the factorization
*
      CALL PCPBTRS( UPLO, N, BW, NRHS, A, JA, DESCA, B, IB, DESCB, WORK,
     $              LWORK-BW*NRHS, WORK( 1+( NB+( 2 )*BW )*BW ),
     $              BW*NRHS, INFO )
*
*     Check info for error conditions
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICTXT, 'PCPBSV', -INFO )
         RETURN
      END IF
*
      RETURN
*
*     End of PCPBSV
*
      END
