      SUBROUTINE PSLACP2( UPLO, M, N, A, IA, JA, DESCA, B, IB, JB,
     $                     DESCB )
*
*  -- ScaLAPACK auxiliary routine (version 1.2) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     May 10, 1996
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            IA, IB, JA, JB, M, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * ), DESCB( * )
      REAL               A( * ), B( * )
*     ..
*
*  Purpose
*  =======
*
*  PSLACP2 copies all or part of a distributed matrix A to another
*  distributed matrix B.  No communication is performed, PSLACP2
*  performs a local copy sub( A ) := sub( B ), where sub( A ) denotes
*  A(IA:IA+M-1,JA:JA+N-1) and sub( B ) denotes B(IB:IB+M-1,JB:JB+N-1).
*  PSLACP2 requires that only dimension of the matrix operands is
*  distributed.
*
*  Notes
*  =====
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN      EXPLANATION
*  --------------- -------------- --------------------------------------
*  DT_A   (global) DESCA( DT_ )   The descriptor type.
*  CTXT_A (global) DESCA( CTXT_ ) The BLACS context handle, indicating
*                                 the BLACS process grid A is distribu-
*                                 ted over. The context itself is glo-
*                                 bal, but the handle (the integer
*                                 value) may vary.
*  M_A    (global) DESCA( M_ )    The number of rows in the distributed
*                                 matrix A.
*  N_A    (global) DESCA( N_ )    The number of columns in the distri-
*                                 buted matrix A.
*  MB_A   (global) DESCA( MB_ )   The blocking factor used to distribute
*                                 the rows of A.
*  NB_A   (global) DESCA( NB_ )   The blocking factor used to distribute
*                                 the columns of A.
*  RSRC_A (global) DESCA( RSRC_ ) The process row over which the first
*                                 row of the matrix A is distributed.
*  CSRC_A (global) DESCA( CSRC_ ) The process column over which the
*                                 first column of A is distributed.
*  LLD_A  (local)  DESCA( LLD_ )  The leading dimension of the local
*                                 array storing the local blocks of the
*                                 distributed matrix A.
*                                 LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*  Arguments
*  =========
*
*  UPLO    (global input) CHARACTER
*          Specifies the part of the distributed matrix sub( A ) to be
*          copied:
*          = 'U':   Upper triangular part is copied; the strictly
*                   lower triangular part of sub( A ) is not referenced;
*          = 'L':   Lower triangular part is copied; the strictly
*                   upper triangular part of sub( A ) is not referenced;
*          Otherwise:  All of the matrix sub( A ) is copied.
*
*  M       (global input) INTEGER
*          The number of rows to be operated on i.e the number of rows
*          of the distributed submatrix sub( A ). M >= 0.
*
*  N       (global input) INTEGER
*          The number of columns to be operated on i.e the number of
*          columns of the distributed submatrix sub( A ). N >= 0.
*
*  A       (local input) REAL pointer into the local memory
*          to an array of dimension (LLD_A, LOCq(JA+N-1) ). This array
*          contains the local pieces of the distributed matrix sub( A )
*          to be copied from.
*
*  IA      (global input) INTEGER
*          A's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension DLEN_.
*          The array descriptor for the distributed matrix A.
*
*  B       (local output) REAL pointer into the local memory
*          to an array of dimension (LLD_B, LOCq(JB+N-1) ). This array
*          contains on exit the local pieces of the distributed matrix
*          sub( B ) set as follows:
*
*          if UPLO = 'U', B(IB+i-1,JB+j-1) = A(IA+i-1,JA+j-1),
*                         1<=i<=j, 1<=j<=N;
*          if UPLO = 'L', B(IB+i-1,JB+j-1) = A(IA+i-1,JA+j-1),
*                         j<=i<=M, 1<=j<=N;
*          otherwise,     B(IB+i-1,JB+j-1) = A(IA+i-1,JA+j-1),
*                         1<=i<=M, 1<=j<=N.
*
*  IB      (global input) INTEGER
*          B's global row index, which points to the beginning of the
*          submatrix which is to be operated on.
*
*  JB      (global input) INTEGER
*          B's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCB   (global and local input) INTEGER array of dimension DLEN_.
*          The array descriptor for the distributed matrix B.
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            BLOCK_CYCLIC_2D, CSRC_, CTXT_, DLEN_, DT_,
     $                   LLD_, MB_, M_, NB_, N_, RSRC_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $                     CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                     RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
      INTEGER            HEIGHT, IACOL, IAROW, IBASEA, IBASEB, IBCOL,
     $                   IBROW, ICOFFA, ICOFFB, IIA, IIAA, IIB, IIBB,
     $                   IIBEGA, IIBEGB, IIENDA, IIENDB, IINXTA, IINXTB,
     $                   ILEFTA, ILEFTB, IRIGHTA, IRIGHTB, IROFFA,
     $                   IROFFB, ITOPA, ITOPB, JJA, JJAA, JJB, JJBB,
     $                   JJBEGA, JJBEGB, JJENDA, JJENDB, JJNXTA, JJNXTB,
     $                   LDA, LDB, MBA, MBB, MPA, MPAA, MPB, MPBB,
     $                   MYCOL, MYDIST, MYROW, NBA, NBB, NPCOL, NPROW,
     $                   NQA, NQAA, NQB, NQBB, WIDE
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, INFOG2L, SLACPY
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            ICEIL, NUMROC
      EXTERNAL           ICEIL, LSAME, NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          MAX, MIN, MOD
*     ..
*     .. Executable Statements ..
*
      IF( M.EQ.0 .OR. N.EQ.0 )
     $   RETURN
*
*     Get grid parameters
*
      CALL BLACS_GRIDINFO( DESCA( CTXT_ ), NPROW, NPCOL, MYROW, MYCOL )
*
      CALL INFOG2L( IA, JA, DESCA, NPROW, NPCOL, MYROW, MYCOL, IIA, JJA,
     $              IAROW, IACOL )
      CALL INFOG2L( IB, JB, DESCB, NPROW, NPCOL, MYROW, MYCOL, IIB, JJB,
     $              IBROW, IBCOL )
*
      MBA = DESCA( MB_ )
      NBA = DESCA( NB_ )
      LDA = DESCA( LLD_ )
      IROFFA = MOD( IA-1, MBA )
      ICOFFA = MOD( JA-1, NBA )
      MBB = DESCB( MB_ )
      NBB = DESCB( NB_ )
      LDB = DESCB( LLD_ )
      IROFFB = MOD( IB-1, MBB )
      ICOFFB = MOD( JB-1, NBB )
*
      IF( N.LE.( NBA-ICOFFA ) ) THEN
*
*        It is assumed that the local columns JJA:JJA+N-1 of the matrix
*        A are in the same process column (IACOL).
*
*                         N
*                JJA             JJA+N-1
*         /      ---------------------    \
*   IROFFA|      |                   |    |
*         \      |...................|    |          ( IAROW )
*           IIA  |x                  |    |   MBA = DESCA( MB_ )
*                | x                 |    |
*                |--x----------------|    /
*                |   x               |
*                |    x              |        ITOP
*                |     x             |          |
*                |      x            |      /-------\
*                |-------x-----------|      |-------x-----------|
*                |        x          |      |        x          |
*                |         x         |      |         x         |
*                |          x        |      |          x        |
*                |           x       |      |           x       |
*                |------------x------|      |------------x------|
*                |             x     |      \____________/
*                |              x    |            |
*                |               x   |          IBASE
*                |                x  |
*                |-----------------x-|          Local picture
*                |                  x|
*                |                   |
*                |                   |
*                |                   |
*                |-------------------|
*                |                   |
*                .                   .
*                .                   .
*                .      (IACOL)      .
*
         IF( MYCOL.EQ.IACOL ) THEN
*
            MPA = NUMROC( M+IROFFA, MBA, MYROW, IAROW, NPROW )
            MPB = NUMROC( M+IROFFB, MBB, MYROW, IBROW, NPROW )
            IF( MPA.LE.0 .OR. MPB.LE.0 )
     $         RETURN
            IF( MYROW.EQ.IAROW )
     $         MPA = MPA - IROFFA
            IF( MYROW.EQ.IBROW )
     $         MPB = MPB - IROFFB
            MYDIST = MOD( MYROW-IAROW+NPROW, NPROW )
            ITOPA = MYDIST * MBA - IROFFA
            ITOPB = MYDIST * MBB - IROFFB
*
            IF( LSAME( UPLO, 'U' ) ) THEN
*
               ITOPA = MAX( 0, ITOPA )
               IIBEGA = IIA
               IIENDA = IIA + MPA - 1
               IINXTA = MIN( ICEIL( IIBEGA, MBA ) * MBA, IIENDA )
               ITOPB = MAX( 0, ITOPB )
               IIBEGB = IIB
               IIENDB = IIB + MPB - 1
               IINXTB = MIN( ICEIL( IIBEGB, MBB ) * MBB, IIENDB )
*
   10          CONTINUE
               IF( ( N-ITOPA ).GT.0 ) THEN
                  CALL SLACPY( UPLO, IINXTA-IIBEGA+1, N-ITOPA,
     $                         A( IIBEGA+(JJA+ITOPA-1)*LDA ), LDA,
     $                         B( IIBEGB+(JJB+ITOPB-1)*LDB ), LDB )
                  MYDIST = MYDIST + NPROW
                  ITOPA = MYDIST * MBA - IROFFA
                  IIBEGA = IINXTA +1
                  IINXTA = MIN( IINXTA+MBA, IIENDA )
                  ITOPB = MYDIST * MBB - IROFFB
                  IIBEGB = IINXTB +1
                  IINXTB = MIN( IINXTB+MBB, IIENDB )
                  GO TO 10
               END IF
*
            ELSE IF( LSAME( UPLO, 'L' ) ) THEN
*
               IIAA = IIA
               JJAA = JJA
               MPAA = MPA
               IBASEA = MIN( ITOPA+MBA, N )
               ITOPA = MIN( MAX( 0, ITOPA ), N )
               IIBB = IIB
               JJBB = JJB
               MPBB = MPB
               IBASEB = MIN( ITOPB+MBB, N )
               ITOPB = MIN( MAX( 0, ITOPB ), N )
*
   20          CONTINUE
               IF( JJAA.LE.( JJA+N-1 ) ) THEN
                  HEIGHT = IBASEA - ITOPA
                  CALL SLACPY( 'All', MPAA, ITOPA-JJAA+JJA,
     $                         A( IIAA+(JJAA-1)*LDA ), LDA,
     $                         B( IIBB+(JJBB-1)*LDB ), LDB )
                  CALL SLACPY( UPLO, MPAA, HEIGHT,
     $                         A( IIAA+(JJA+ITOPA-1)*LDA ), LDA,
     $                         B( IIBB+(JJB+ITOPB-1)*LDB ), LDB )
                  MPAA = MAX( 0, MPAA - HEIGHT )
                  IIAA = IIAA + HEIGHT
                  JJAA = JJA + IBASEA
                  MPBB = MAX( 0, MPBB - HEIGHT )
                  IIBB = IIBB + HEIGHT
                  JJBB = JJB + IBASEB
                  MYDIST = MYDIST + NPROW
                  ITOPA = MYDIST * MBA - IROFFA
                  IBASEA = MIN( ITOPA + MBA, N )
                  ITOPA = MIN( ITOPA, N )
                  ITOPB = MYDIST * MBB - IROFFB
                  IBASEB = MIN( ITOPB + MBB, N )
                  ITOPB = MIN( ITOPB, N )
                  GO TO 20
               END IF
*
            ELSE
*
               IIAA = IIA
               JJAA = JJA
               MPAA = MPA
               IBASEA = MIN( ITOPA+MBA, N )
               ITOPA = MIN( MAX( 0, ITOPA ), N )
               IIBB = IIB
               JJBB = JJB
               MPBB = MPB
               IBASEB = MIN( ITOPB+MBB, N )
               ITOPB = MIN( MAX( 0, ITOPB ), N )
*
   30          CONTINUE
               IF( JJAA.LE.( JJA+N-1 ) ) THEN
                  HEIGHT = IBASEA - ITOPA
                  CALL SLACPY( 'All', MPA, ITOPA-JJAA+JJA,
     $                         A( IIA+(JJAA-1)*LDA ), LDA,
     $                         B( IIB+(JJBB-1)*LDB ), LDB )
                  CALL SLACPY( 'All', MPA-MPAA, HEIGHT,
     $                         A( IIA+(JJA+ITOPA-1)*LDA ), LDA,
     $                         B( IIB+(JJB+ITOPB-1)*LDB ), LDB )
                  CALL SLACPY( 'All', MPAA, HEIGHT,
     $                         A( IIAA+(JJA+ITOPA-1)*LDA ), LDA,
     $                         B( IIBB+(JJB+ITOPB-1)*LDB ), LDB )
                  MPAA = MAX( 0, MPAA - HEIGHT )
                  IIAA = IIAA + HEIGHT
                  JJAA = JJA + IBASEA
                  MPBB = MAX( 0, MPBB - HEIGHT )
                  IIBB = IIBB + HEIGHT
                  JJBB = JJB + IBASEB
                  MYDIST = MYDIST + NPROW
                  ITOPA = MYDIST * MBA - IROFFA
                  IBASEA = MIN( ITOPA + MBA, N )
                  ITOPA = MIN( ITOPA, N )
                  ITOPB = MYDIST * MBB - IROFFB
                  IBASEB = MIN( ITOPB + MBB, N )
                  ITOPB = MIN( ITOPB, N )
                  GO TO 30
               END IF
*
            END IF
*
         END IF
*
      ELSE IF( M.LE.( MBA-IROFFA ) ) THEN
*
*        It is assumed that the local rows IIA:IIA+M-1 of the matrix A
*        are in the same process row (IAROW).
*
*            ICOFFA
*             / \JJA
*        IIA  ------------------ ....            --------
*             | .x  |    |    |                 / |    | \
*             | . x |    |    |            ILEFT| |    | |
*             | .  x     |    |                 | |    | |
*             | .   x    |    |                 \ x    | |
*             | .   |x   |    |                   |x   | | IRIGHT
*             | .   | x  |    |                   | x  | |
*    (IAROW)  | .   |  x |    |                   |  x | |
*             | .   |   x|    |                   |   x| |
*             | .   |    x    |                   |    x /
*             | .   |    |x   |                   |    |
*             | .   |    | x  |                   |    |
*             | .   |    |  x |                   |    |
*             | .   |    |   x|                   |    |
*    IIA+M-1  ------------------ ....            -------
*              NB_A
*             (IACOL)                          Local picture
*
         IF( MYROW.EQ.IAROW ) THEN
*
            NQA = NUMROC( N+ICOFFA, NBA, MYCOL, IACOL, NPCOL )
            NQB = NUMROC( N+ICOFFB, NBB, MYCOL, IBCOL, NPCOL )
            IF( NQA.LE.0 .OR. NQB.LE.0 )
     $         RETURN
            IF( MYCOL.EQ.IACOL )
     $         NQA = NQA - ICOFFA
            IF( MYCOL.EQ.IBCOL )
     $         NQB = NQB - ICOFFB
            MYDIST = MOD( MYCOL-IACOL+NPCOL, NPCOL )
            ILEFTA = MYDIST * NBA - ICOFFA
            ILEFTB = MYDIST * NBB - ICOFFB
*
            IF( LSAME( UPLO, 'L' ) ) THEN
*
               ILEFTA = MAX( 0, ILEFTA )
               JJBEGA = JJA
               JJENDA = JJA + NQA - 1
               JJNXTA = MIN( ICEIL( JJBEGA, NBA ) * NBA, JJENDA )
               ILEFTB = MAX( 0, ILEFTB )
               JJBEGB = JJB
               JJENDB = JJB + NQB - 1
               JJNXTB = MIN( ICEIL( JJBEGB, NBB ) * NBB, JJENDB )
*
   40          CONTINUE
               IF( ( M-ILEFTA ).GT.0 ) THEN
                  CALL SLACPY( UPLO, M-ILEFTA, JJNXTA-JJBEGA+1,
     $                         A( IIA+ILEFTA+(JJBEGA-1)*LDA ), LDA,
     $                         B( IIB+ILEFTB+(JJBEGB-1)*LDB ), LDB )
                  MYDIST = MYDIST + NPCOL
                  ILEFTA = MYDIST * NBA - ICOFFA
                  JJBEGA = JJNXTA +1
                  JJNXTA = MIN( JJNXTA+NBA, JJENDA )
                  ILEFTB = MYDIST * NBB - ICOFFB
                  JJBEGB = JJNXTB +1
                  JJNXTB = MIN( JJNXTB+NBB, JJENDB )
                  GO TO 40
               END IF
*
            ELSE IF( LSAME( UPLO, 'U' ) ) THEN
*
               IIAA = IIA
               JJAA = JJA
               NQAA = NQA
               IRIGHTA = MIN( ILEFTA+NBA, M )
               ILEFTA = MIN( MAX( 0, ILEFTA ), M )
               IIBB = IIB
               JJBB = JJB
               NQBB = NQB
               IRIGHTB = MIN( ILEFTB+NBB, M )
               ILEFTB = MIN( MAX( 0, ILEFTB ), M )
*
   50          CONTINUE
               IF( IIAA.LE.( IIA+M-1 ) ) THEN
                  WIDE = IRIGHTA - ILEFTA
                  CALL SLACPY( 'All', ILEFTA-IIAA+IIA, NQAA,
     $                         A( IIAA+(JJAA-1)*LDA ), LDA,
     $                         B( IIBB+(JJBB-1)*LDB ), LDB )
                  CALL SLACPY( UPLO, WIDE, NQAA,
     $                         A( IIA+ILEFTA+(JJAA-1)*LDA ), LDA,
     $                         B( IIB+ILEFTB+(JJBB-1)*LDB ), LDB )
                  NQAA = MAX( 0, NQAA - WIDE )
                  IIAA = IIA + IRIGHTA
                  JJAA = JJAA + WIDE
                  NQBB = MAX( 0, NQBB - WIDE )
                  IIBB = IIB + IRIGHTB
                  JJBB = JJBB + WIDE
                  MYDIST = MYDIST + NPCOL
                  ILEFTA = MYDIST * NBA - ICOFFA
                  IRIGHTA = MIN( ILEFTA + NBA, M )
                  ILEFTA = MIN( ILEFTA, M )
                  ILEFTB = MYDIST * NBB - ICOFFB
                  IRIGHTB = MIN( ILEFTB + NBB, M )
                  ILEFTB = MIN( ILEFTB, M )
                  GO TO 50
               END IF
*
            ELSE
*
               IIAA = IIA
               JJAA = JJA
               NQAA = NQA
               IRIGHTA = MIN( ILEFTA+NBA, M )
               ILEFTA = MIN( MAX( 0, ILEFTA ), M )
               IIBB = IIB
               JJBB = JJB
               NQBB = NQB
               IRIGHTB = MIN( ILEFTB+NBB, M )
               ILEFTB = MIN( MAX( 0, ILEFTB ), M )
*
   60          CONTINUE
               IF( IIAA.LE.( IIA+M-1 ) ) THEN
                  WIDE = IRIGHTA - ILEFTA
                  CALL SLACPY( 'All', ILEFTA-IIAA+IIA, NQA,
     $                         A( IIAA+(JJA-1)*LDA ), LDA,
     $                         B( IIBB+(JJB-1)*LDB ), LDB )
                  CALL SLACPY( 'All', WIDE, NQA-NQAA,
     $                         A( IIA+ILEFTA+(JJA-1)*LDA ), LDA,
     $                         B( IIB+ILEFTB+(JJB-1)*LDB ), LDB )
                  CALL SLACPY( 'All', WIDE, NQAA,
     $                         A( IIA+ILEFTA+(JJAA-1)*LDA ), LDA,
     $                         B( IIB+ILEFTB+(JJBB-1)*LDB ), LDB )
                  NQAA = MAX( 0, NQAA - WIDE )
                  IIAA = IIA + IRIGHTA
                  JJAA = JJAA + WIDE
                  NQBB = MAX( 0, NQBB - WIDE )
                  IIBB = IIB + IRIGHTB
                  JJBB = JJBB + WIDE
                  MYDIST = MYDIST + NPCOL
                  ILEFTA = MYDIST * NBA - ICOFFA
                  IRIGHTA = MIN( ILEFTA + NBA, M )
                  ILEFTA = MIN( ILEFTA, M )
                  ILEFTB = MYDIST * NBB - ICOFFB
                  IRIGHTB = MIN( ILEFTB + NBB, M )
                  ILEFTB = MIN( ILEFTB, M )
                  GO TO 60
               END IF
*
            END IF
*
         END IF
*
      END IF
*
      RETURN
*
*     End of PSLACP2
*
      END
