      SUBROUTINE PZPBTRF( UPLO, N, BW, A, JA, DESCA, AF, LAF, WORK,
     $                    LWORK, INFO )
*
*  -- ScaLAPACK routine (version 1.3 ALPHA) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     June 5, 1996
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            BW, INFO, JA, LAF, LWORK, N
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * )
      COMPLEX*16         A( * ), AF( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PZPBTRF computes the Cholesky factorization of an N-by-N cplx
*  banded hermitian positive definite distributed matrix sub( A )
*  with bandwidth BW denoted by
*  A(1:N, JA:JA+N-1).
*
*  The factorization has the form
*
*            sub( A ) = U' * U ,  if UPLO = 'U', or
*
*            sub( A ) = L  * L',  if UPLO = 'L',
*
*  where U is a banded upper triangular matrix and L is banded lower
*  triangular.
*
*  =====================================================================
*
*  Arguments
*  =========
*
*  UPLO    (global input) CHARACTER
*          = 'U':  Upper triangle of sub( A ) is stored;
*          = 'L':  Lower triangle of sub( A ) is stored.
*
*  N       (global input) INTEGER
*          The number of rows and columns to be operated on, i.e. the
*          order of the distributed submatrix sub( A ). N >= 0.
*
*  BW      Number of subdiagonals
*
*
*  A       (local input/local output) COMPLEX*16 pointer into the
*          local memory to an array of dimension with first dimension
*          LLD_A.
*          On entry, this array contains the local pieces of the
*          N-by-N symmetric banded distributed matrix sub( A ) to be
*          factored.
*          This local portion is stored in the packed banded format
*            used in LAPACK.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension >=7
*          if 1D type, >=9 if 2D type.
*          The array descriptor for the distributed matrix A.
*
*  INFO    (global output) INTEGER
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*          > 0:  If INFO = K, the leading minor of order K,
*                A(IA:IA+K-1,JA:JA+K-1) is not positive definite, and
*                the factorization could not be completed.
*
*  =====================================================================
*
*
*  Restrictions
*  ============
*
*  The following are restrictions on the input parameters. Many of these
*    are temporary and will be removed in future releases, while others
*    may reflect fundamental technical limitations.
*
*    Non-cyclic restriction: VERY IMPORTANT!
*      P*NB<= JA+N.
*      The mapping for matrices must be blocked, reflecting the nature
*      of the divide and conquer algorithm as a task-parallel algorithm.
*
*    Blocksize cannot be too small:
*      NB >= 2*BW
*      The bulk of parallel computation is done on the matrix of size
*      NB-BW on each processor. If this is too small, divide and conquer
*      is a poor choice of algorithm.
*
*    Submatrix reference:
*      JA = IB
*      Alignment restriction that prevents unnecessary communication.
*
*
*  =====================================================================
*
*
*  Notes
*  =====
*
*  If the factorization routine and the solve routine are to be called
*    separately (to solve various sets of righthand sides using the same
*    coefficient matrix), the auxiliary space AF *must not be altered*
*    between calls to the factorization routine and the solve routine.
*
*  The best algorithm for solving banded and tridiagonal linear systems
*    depends on a variety of parameters, especially the bandwidth.
*    Currently, only algorithms designed for the case N/P >> bw are
*    implemented. These go by many names, including Divide and Conquer,
*    Partitioning, domain decomposition-type, etc.
*
*  Algorithm description: Divide and Conquer
*
*    The Divide and Conqer algorithm assumes the matrix is narrowly
*      banded compared with the number of equations. In this situation,
*      it is best to store the matrix one-dimensionally.
*      The basic algorithm divides the banded matrix up into a small
*      number of pieces (usually P with one stored on each processor)
*      and then proceeds in 2 phases for the factorization or 3 for the
*      solution of a linear system.
*      1) Local Phase:
*         The individual pieces are factored independently and in
*         parallel. These factors are applied to the matrix creating
*         fillin. For a linear system, a local front solve is
*         performed in this stage.
*      2) Reduced System Phase:
*         A small ( BW* (P-1) ) matrix/system is formed representing the
*         interaction of the larger blocks. It can be factored in
*         several ways. Currently a parallel Block Cyclic Reduction
*         algorithm is used. For a linear system, a parallel front solve
*         followed by an analagous backsolve, both using the structure
*         of the factored matrix, are performed.
*      3) Backsubsitution Phase:
*         For a linear system, a local backsubstitution is performed on
*         each processor in parallel.
*
*  Descriptors now have *types* and differ from ScaLAPACK 1.0.
*
*  Banded codes can use either the standard two-dimensional descriptors
*    or new one-dimensional descriptors, though the processor grid in
*    both cases *must be one-dimensional*. We describe both types below.
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN      EXPLANATION
*  --------------- -------------- --------------------------------------
*  DT_A   (global) DESCA( DT_ )   The descriptor type.
*  CTXT_A (global) DESCA( CTXT_ ) The BLACS context handle, indicating
*                                 the BLACS process grid A is distribu-
*                                 ted over. The context itself is glo-
*                                 bal, but the handle (the integer
*                                 value) may vary.
*  M_A    (global) DESCA( M_ )    The number of rows in the distributed
*                                 matrix A.
*  N_A    (global) DESCA( N_ )    The number of columns in the distri-
*                                 buted matrix A.
*  MB_A   (global) DESCA( MB_ )   The blocking factor used to distribute
*                                 the rows of A.
*  NB_A   (global) DESCA( NB_ )   The blocking factor used to distribute
*                                 the columns of A.
*  RSRC_A (global) DESCA( RSRC_ ) The process row over which the first
*                                 row of the matrix A is distributed.
*  CSRC_A (global) DESCA( CSRC_ ) The process column over which the
*                                 first column of A is distributed.
*  LLD_A  (local)  DESCA( LLD_ )  The leading dimension of the local
*                                 array storing the local blocks of the
*                                 distributed matrix A.
*                                 LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*
*  One-dimensional descriptors:
*
*  One-dimensional descriptors are a new addition to ScaLAPACK since
*    version 1.0. They simplify and shorten the descriptor for 1D
*    matrices. Such a descriptor is associated with each one-
*    dimensionally distributed matrix in ScaLAPACK. This descriptor
*    allows ScaLAPACK programs to be addressed in global indices by
*    providing the means to translate between global indices and local
*    indices in a way that insulates the user from such details.
*
*  Since ScaLAPACK supports two-dimensional matrices as the fundamental
*    object, we allow for matrices to be distributed either over the
*    first dimension (as if the grid were P-by-1) or the second
*    dimension (as if the grid were 1-by-P). This choice is made in the
*    first entry of the descriptor as described below.
*    IMPORTANT NOTE: the actual BLACS grid represented by the
*    CTXT entry in the descriptor may be *either*  P-by-1 or 1-by-P
*    for either one-dimensional descriptor type. The ScaLAPACK
*    routines will interpret these properly depending on the TYPE.
*    ScaLAPACK routines *do not support intercontext operations* so that
*    the grid passed to a single ScaLAPACK routine *must be the same*
*    for all matrix descriptors passed to that routine.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  TYPE_A (global) DESCA( 1 ) The orientation of the one-dimensional
*                             grid.
*                             TYPE_A = 502: P-by-1 grid.
*                             TYPE_A = 501: 1-by-P grid.
*  CTXT_A (global) DESCA( 2 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the
*                             handle (the integer value) may vary.
*  N_A    (global) DESCA( 3 ) The size of the matrix dimension being
*                             distributed.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the distributed dimension of the matrix.
*  SRC_A  (global) DESCA( 5 ) The process row or column over which the
*                             1st element of the matrix is distributed.
*  LLD_A  (local)  DESCA( 6 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. Minimum value of LLD_A
*                             depends on TYPE_A.
*  Reserved        DESCA( 7 ) Reserved for future use.
*
*
*  =====================================================================
*
*  Implemented by: Andrew J. Cleary, June 5, 1996
*
*  =====================================================================
*
*     .. Parameters ..
      DOUBLE PRECISION   ONE
      PARAMETER          ( ONE = 1.0D+0 )
      DOUBLE PRECISION   ZERO
      PARAMETER          ( ZERO = 0.0D+0 )
      COMPLEX*16         CONE
      PARAMETER          ( CONE = ( 1.0D+0, 0.0D+0 ) )
      COMPLEX*16         CZERO
      PARAMETER          ( CZERO = ( 0.0D+0, 0.0D+0 ) )
      INTEGER            INT_ONE
      PARAMETER          ( INT_ONE = 1 )
      INTEGER            DESCMULT, BIGNUM
      PARAMETER          ( DESCMULT = 100, BIGNUM = DESCMULT*DESCMULT )
      INTEGER            BLOCK_CYCLIC_2D, CSRC_, CTXT_, DLEN_, DT_,
     $                   LLD_, MB_, M_, NB_, N_, RSRC_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $                     CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                     RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
      INTEGER            COMM_PROC, CSRC, FIRST_PROC, I, ICTXT,
     $                   ICTXT_NEW, ICTXT_SAVE, IDUM1, JA_NEW, LAF_MIN,
     $                   LEVEL_DIST, LLDA, MYCOL, MYROW, NB,
     $                   NEXT_TRI_SIZE_M, NEXT_TRI_SIZE_N, NP, NPCOL,
     $                   NPROW, NP_SAVE, ODD_SIZE, OFST, PART_OFFSET,
     $                   PART_SIZE, PREV_TRI_SIZE_M, PREV_TRI_SIZE_N,
     $                   RETURN_CODE, STORE_N, WORK_SIZE_MIN
*     ..
*     .. Local Arrays ..
      INTEGER            DESCA_1XP( 7 ), PARAM_CHECK( 9, 3 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDEXIT, BLACS_GRIDINFO,
     $                   DESC_CONVERT, GLOBCHK, PXERBLA, RESHAPE, ZAXPY,
     $                   ZGEMM, ZGERV2D, ZGESD2D, ZHERK, ZLACPY,
     $                   ZLATCPY, ZPBTRF, ZPOTRF, ZTBTRS, ZTRMM,
     $                   ZTRRV2D, ZTRSD2D, ZTRSM, ZTRTRS
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      INTEGER            NUMROC
      EXTERNAL           LSAME, NUMROC
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ICHAR, MIN, MOD
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
*     Test the input parameters
*
      INFO = 0
*
*
*     Convert descriptor into standard form for easy access to
*        parameters, check that grid is of right shape.
*
      DESCA_1XP( 1 ) = 501
*
      CALL DESC_CONVERT( DESCA, DESCA_1XP, RETURN_CODE )
*
      IF( RETURN_CODE.NE.0 ) THEN
         INFO = -( 6*100+2 )
      END IF
*
*
*     Get values out of descriptor for use in code.
*
      ICTXT = DESCA_1XP( 2 )
      CSRC = DESCA_1XP( 5 )
      NB = DESCA_1XP( 4 )
      LLDA = DESCA_1XP( 6 )
      STORE_N = DESCA_1XP( 3 )
*
*     Get grid parameters
*
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
      NP = NPROW*NPCOL
*
*
      IF( LSAME( UPLO, 'U' ) ) THEN
         IDUM1 = ICHAR( 'U' )
      ELSE IF( LSAME( UPLO, 'L' ) ) THEN
         IDUM1 = ICHAR( 'L' )
      ELSE
         INFO = -1
      END IF
*
      IF( N.LT.0 ) THEN
         INFO = -2
      END IF
*
      IF( N+JA-1.LT.STORE_N ) THEN
         INFO = -( 6*100+3 )
      END IF
*
      IF( ( BW.GT.N-1 ) .OR. ( BW.LT.0 ) ) THEN
         INFO = -3
      END IF
*
      IF( LLDA.LT.BW+1 ) THEN
         INFO = -( 6*100+6 )
      END IF
*
*     Argument checking that is specific to Divide & Conquer routine
*
      IF( NPROW.NE.1 ) THEN
         INFO = -( 6*100+2 )
      END IF
*
      IF( N.GT.NP*NB-MOD( JA-1, NB ) ) THEN
         INFO = -( 2 )
         CALL PXERBLA( ICTXT, 'PZPBTRF, D&C alg.: only 1 block per proc'
     $                 , INFO )
         RETURN
      END IF
*
*     Current restrictions on Divide and Conquer that may be lifted
*
      IF( CSRC.NE.0 ) THEN
         INFO = -( 6*100+5 )
      END IF
*
      IF( JA.NE.1 ) THEN
         INFO = -( 5 )
      END IF
*
*
*     Check auxiliary storage size
*
      LAF_MIN = ( NUMROC( N, NB, MYCOL, 0, NPCOL )+( 2 )*BW )*BW
      IF( LAF.LT.LAF_MIN ) THEN
         INFO = -8
*        put minimum value of laf into AF( 1 )
         AF( 1 ) = LAF_MIN
         CALL PXERBLA( ICTXT,
     $                 'PZPBTRF1Dd_and_c: auxiliary storage error ',
     $                 -INFO )
         RETURN
      END IF
*
*     Check worksize
*
      WORK_SIZE_MIN = BW*BW
      IF( LWORK.LT.WORK_SIZE_MIN ) THEN
         INFO = -10
*        put minimum value of work into work( 1 )
         WORK( 1 ) = WORK_SIZE_MIN
         CALL PXERBLA( ICTXT, 'PZPBTRF1Dd_and_c: worksize error ',
     $                 -INFO )
         RETURN
      END IF
*
*     Pack params and positions into arrays for global consistency check
*
      PARAM_CHECK( 9, 1 ) = DESCA( 5 )
      PARAM_CHECK( 8, 1 ) = DESCA( 4 )
      PARAM_CHECK( 7, 1 ) = DESCA( 3 )
      PARAM_CHECK( 6, 1 ) = DESCA( 2 )
      PARAM_CHECK( 5, 1 ) = DESCA( 1 )
      PARAM_CHECK( 4, 1 ) = JA
      PARAM_CHECK( 3, 1 ) = BW
      PARAM_CHECK( 2, 1 ) = N
      PARAM_CHECK( 1, 1 ) = IDUM1
*
      PARAM_CHECK( 9, 2 ) = 605
      PARAM_CHECK( 8, 2 ) = 604
      PARAM_CHECK( 7, 2 ) = 603
      PARAM_CHECK( 6, 2 ) = 602
      PARAM_CHECK( 5, 2 ) = 601
      PARAM_CHECK( 4, 2 ) = 5
      PARAM_CHECK( 3, 2 ) = 3
      PARAM_CHECK( 2, 2 ) = 2
      PARAM_CHECK( 1, 2 ) = 1
*
*     Want to find errors with MIN( ), so if no error, set it to a big
*     number. If there already is an error, multiply by the the
*     descriptor multiplier.
*
      IF( INFO.GE.0 ) THEN
         INFO = BIGNUM
      ELSE IF( INFO.LT.-DESCMULT ) THEN
         INFO = -INFO
      ELSE
         INFO = -INFO*DESCMULT
      END IF
*
*     Check consistency across processors
*
      CALL GLOBCHK( ICTXT, 9, PARAM_CHECK, 9, PARAM_CHECK( 1, 3 ),
     $              INFO )
*
*     Prepare output: set info = 0 if no error, and divide by DESCMULT
*     if error is not in a descriptor entry.
*
      IF( INFO.EQ.BIGNUM ) THEN
         INFO = 0
      ELSE IF( MOD( INFO, DESCMULT ).EQ.0 ) THEN
         INFO = -INFO / DESCMULT
      ELSE
         INFO = -INFO
      END IF
*
      IF( INFO.LT.0 ) THEN
         CALL PXERBLA( ICTXT, 'PZPBTRF', -INFO )
         RETURN
      END IF
*
*
*     Form a new BLACS grid (the "standard form" grid) with only procs
*        holding part of the matrix, of size 1xNP where NP is adjusted,
*        starting at csrc=0, with JA modified to reflext dropped procs.
*
*     First processor to hold part of the matrix:
*
      FIRST_PROC = MOD( ( JA-1 ) / NB+CSRC, NPCOL )
*
*     Calculate new JA one while dropping off unused processors.
*
      JA_NEW = MOD( JA-1, NB ) + 1
*
*     Save and compute new value of NP
*
      NP_SAVE = NP
      NP = ( JA_NEW+N-2 ) / NB + 1
*
*     Call utility routine that forms "standard-form" grid
*
      CALL RESHAPE( ICTXT, INT_ONE, ICTXT_NEW, INT_ONE, FIRST_PROC,
     $              INT_ONE, NP )
*
*     Use new context from standard grid as context.
*
      ICTXT_SAVE = ICTXT
      ICTXT = ICTXT_NEW
*
*     Get information about new grid.
*
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
*
*     Drop out processors that do not have part of the matrix.
*
      IF( MYROW.LT.0 ) THEN
         GO TO 100
      END IF
*
*     ********************************
*     Values reused throughout routine
*
*     User-input value of partition size
*
      PART_SIZE = NB
*
*     Size of main (or odd) partition in each processor
*
      ODD_SIZE = NUMROC( N, PART_SIZE, MYCOL, 0, NPCOL )
      IF( MYCOL.LT.NP-1 ) THEN
         ODD_SIZE = ODD_SIZE - BW
      END IF
*
*     Offset to beginning of main (or odd) partition in each processor
*
      PART_OFFSET = 0
      IF( MYCOL.EQ.0 ) THEN
         PART_OFFSET = MOD( JA_NEW-1, PART_SIZE )
      END IF
*
      OFST = PART_OFFSET*LLDA
*
*
*       Zero out space for fillin
*
      DO 10 I = 1, LAF_MIN
         AF( I ) = CZERO
   10 CONTINUE
*
*       Zero out space for work
*
      DO 20 I = 1, WORK_SIZE_MIN
         WORK( I ) = CZERO
   20 CONTINUE
*
*     Begin main code
*
      IF( LSAME( UPLO, 'L' ) ) THEN
*
********************************************************************
*       PHASE 1: Local computation phase.
********************************************************************
*
*
*       Sizes of the extra triangles communicated bewtween processors
*
         IF( MYCOL.GT.0 ) THEN
            PREV_TRI_SIZE_M = MIN( BW, NUMROC( N, PART_SIZE, MYCOL, 0,
     $                        NPCOL ) )
            PREV_TRI_SIZE_N = MIN( BW, NUMROC( N, PART_SIZE, MYCOL-1, 0,
     $                        NPCOL ) )
         END IF
*
         IF( MYCOL.LT.NPCOL-1 ) THEN
            NEXT_TRI_SIZE_M = MIN( BW, NUMROC( N, PART_SIZE, MYCOL+1, 0,
     $                        NPCOL ) )
            NEXT_TRI_SIZE_N = MIN( BW, NUMROC( N, PART_SIZE, MYCOL, 0,
     $                        NPCOL ) )
         END IF
*
         IF( MYCOL.LT.NP-1 ) THEN
*         Transfer last triangle D_i of local matrix to next processor
*         which needs it to calculate fillin due to factorization of
*         its main (odd) block A_i.
*         Overlap the send with the factorization of A_i.
*
            CALL ZTRSD2D( ICTXT, 'U', 'N', NEXT_TRI_SIZE_M,
     $                    NEXT_TRI_SIZE_N, A( OFST+ODD_SIZE*LLDA+( BW+
     $                    1 ) ), LLDA-1, 0, MYCOL+1 )
*
         END IF
*
*
*       Factor main partition A_i = L_i {L_i}^T in each processor
*
         CALL ZPBTRF( UPLO, ODD_SIZE, BW, A( OFST+1 ), LLDA, INFO )
*
         IF( INFO.NE.0 ) THEN
            INFO = INFO + NB*MYCOL
            GO TO 90
         END IF
*
*
         IF( MYCOL.LT.NP-1 ) THEN
*         Apply factorization to odd-even connection block B_i
*
*         Transpose the connection block in preparation.
*
            CALL ZLATCPY( 'U', BW, BW, A( ( OFST+( BW+1 )+( ODD_SIZE-
     $                    BW )*LLDA ) ), LLDA-1,
     $                    AF( ( ODD_SIZE+2*BW )*BW+1 ), BW )
*
*         Perform the triangular system solve {L_i}{{B'}_i}^T = {B_i}^T
*
            CALL ZTRTRS( UPLO, 'N', 'N', BW, BW,
     $                   A( OFST+1+( ODD_SIZE-BW )*LLDA ), LLDA-1,
     $                   AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, INFO )
            IF( INFO.NE.0 ) THEN
               INFO = INFO + NB*MYCOL + ODD_SIZE
               GO TO 90
            END IF
*
*         Transpose resulting block to its location in main storage.
*
            CALL ZLATCPY( 'L', BW, BW, AF( ( ODD_SIZE+2*BW )*BW+1 ), BW,
     $                    A( ( OFST+( BW+1 )+( ODD_SIZE-BW )*LLDA ) ),
     $                    LLDA-1 )
*
*
*         Compute contribution to diagonal block of reduced system.
*          {C'}_i = {C_i}-{{B'}_i}{{B'}_i}^T
*
*         The following method uses more flops than necessary but
*           does not necessitate the writing of a new BLAS routine.
*
*
            CALL ZHERK( UPLO, 'C', BW, BW, -ONE,
     $                  AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, ONE,
     $                  A( OFST+1+ODD_SIZE*LLDA ), LLDA-1 )
*
         END IF
*       End of "if ( MYCOL .lt. NP-1 )..." loop
*
*
*
         IF( MYCOL.NE.0 ) THEN
*         Discard temporary matrix stored beginning in
*           AF( (odd_size+2*bw)*bw+1 ) and use for
*           off_diagonal block of reduced system.
*         Receive previously transmitted matrix section, which forms
*         the right-hand-side for the triangular solve that calculates
*         the "spike" fillin.
*
*
            CALL ZTRRV2D( ICTXT, 'U', 'N', PREV_TRI_SIZE_M,
     $                    PREV_TRI_SIZE_N, AF( 1 ), ODD_SIZE, 0,
     $                    MYCOL-1 )
*
*         Calculate the "spike" fillin, ${L_i} {{G}_i}^T = {D_i}$ .
*
            CALL ZTBTRS( UPLO, 'N', 'N', ODD_SIZE, BW, BW, A( OFST+1 ),
     $                   LLDA, AF( 1 ), ODD_SIZE, INFO )
            IF( INFO.NE.0 ) THEN
               INFO = INFO + NB*MYCOL + ODD_SIZE
               GO TO 90
            END IF
*
*         Calculate the update block for previous proc, E_i = G_i{G_i}^T
*
            CALL ZHERK( 'L', 'C', BW, ODD_SIZE, -ONE, AF( 1 ), ODD_SIZE,
     $                  ZERO, AF( 1+( ODD_SIZE+2*BW )*BW ), BW )
*
*
*         Initiate send of E_i to previous processor to overlap
*           with next computation.
*
            CALL ZGESD2D( ICTXT, BW, BW, AF( ( ODD_SIZE+2*BW )*BW+1 ),
     $                    BW, 0, MYCOL-1 )
*
*
            IF( MYCOL.LT.NP-1 ) THEN
*
*           Calculate off-diagonal block of reduced system.
*           Note: for ease of use in solution of reduced system, store
*           off-diagonal block in transpose form.
*           {F_i}^T =  {H_i}{{B'}_i}^T
*
*           Copy matrix H_i (the last bw cols of G_i) to AF storage
*             as per requirements of BLAS routine ZTRMM.
*             Since we have G_i^T stored, transpose H_i^T to H_i.
*
               CALL ZLATCPY( 'N', BW, BW, AF( ODD_SIZE-BW+1 ), ODD_SIZE,
     $                       AF( ( ODD_SIZE )*BW+1 ), BW )
*
               CALL ZTRMM( 'R', 'U', 'C', 'N', BW, BW, -CONE,
     $                     A( ( OFST+( BW+1 )+( ODD_SIZE-BW )*LLDA ) ),
     $                     LLDA-1, AF( ( ODD_SIZE )*BW+1 ), BW )
*
*
            END IF
*
         END IF
*       End of "if ( MYCOL .ne. 0 )..."
*
********************************************************************
*       PHASE 2: Formation and factorization of Reduced System.
********************************************************************
*
*       Gather up local sections of reduced system
*
*     The last processor does not participate in the factorization of
*       the reduced system, having sent its E_i already.
         IF( MYCOL.EQ.NPCOL-1 ) THEN
            GO TO 50
         END IF
*
*       Initiate send of off-diagonal block to overlap with next part.
*       Off-diagonal block needed on neighboring processor to start
*       algorithm.
*
         IF( ( MOD( MYCOL+1, 2 ).EQ.0 ) .AND. ( MYCOL.GT.0 ) ) THEN
*
            CALL ZGESD2D( ICTXT, BW, BW, AF( ( ODD_SIZE )*BW+1 ), BW, 0,
     $                    MYCOL-1 )
*
         END IF
*
*       Copy last triangular block into AF storage for subsequent
*         operations.
*
         CALL ZLACPY( 'L', BW, BW, A( OFST+ODD_SIZE*LLDA+1 ), LLDA-1,
     $                AF( ( ODD_SIZE+1*BW )*BW+1 ), BW )
*
*       Receive cont. to diagonal block that is stored on this proc.
*
         IF( MYCOL.LT.NPCOL-1 ) THEN
*
            CALL ZGERV2D( ICTXT, BW, BW,
     $                    AF( ( ODD_SIZE+( 2 )*BW )*BW+1 ), BW, 0,
     $                    MYCOL+1 )
*
*          Add contribution to diagonal block
*
            CALL ZAXPY( BW*BW, CONE, AF( ( ODD_SIZE+( 2 )*BW )*BW+1 ),
     $                  1, AF( ( ODD_SIZE+BW )*BW+1 ), 1 )
*
         END IF
*
*
*       *************************************
*       Modification Loop
*
*       The distance for sending and receiving for each level starts
*         at 1 for the first level.
         LEVEL_DIST = 1
*
*       Do until this proc is needed to modify other procs' equations
*
   30    CONTINUE
         IF( MOD( ( MYCOL+1 ) / LEVEL_DIST, 2 ).NE.0 )
     $      GO TO 40
*
*         Receive and add contribution to diagonal block from the left
*
         IF( MYCOL-LEVEL_DIST.GE.0 ) THEN
            CALL ZGERV2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL-LEVEL_DIST )
*
            CALL ZAXPY( BW*BW, CONE, WORK( 1 ), 1,
     $                  AF( ( ODD_SIZE+BW )*BW+1 ), 1 )
*
         END IF
*
*         Receive and add contribution to diagonal block from the right
*
         IF( MYCOL+LEVEL_DIST.LT.NPCOL-1 ) THEN
            CALL ZGERV2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL+LEVEL_DIST )
*
            CALL ZAXPY( BW*BW, CONE, WORK( 1 ), 1,
     $                  AF( ( ODD_SIZE+BW )*BW+1 ), 1 )
*
         END IF
*
         LEVEL_DIST = LEVEL_DIST*2
*
         GO TO 30
   40    CONTINUE
*       [End of GOTO Loop]
*
*
*       *********************************
*       Calculate and use this proc's blocks to modify other procs'...
*
*       Factor diagonal block
*
         CALL ZPOTRF( 'L', BW, AF( ( ODD_SIZE+BW )*BW+1 ), BW, INFO )
*
         IF( INFO.NE.0 ) THEN
            INFO = INFO + N - ( NPCOL-1 )*BW + MYCOL*BW
            GO TO 90
         END IF
*
*       ****************************************************************
*       Receive offdiagonal block from processor to right.
*         If this is the first group of processors, the receive comes
*         from a different processor than otherwise.
*
         IF( LEVEL_DIST.EQ.1 ) THEN
            COMM_PROC = MYCOL + 1
*
*           Move block into place that it will be expected to be for
*             calcs.
*
            CALL ZLACPY( 'N', BW, BW, AF( ( ODD_SIZE )*BW+1 ), BW,
     $                   AF( ( ODD_SIZE+2*BW )*BW+1 ), BW )
*
         ELSE
            COMM_PROC = MYCOL + LEVEL_DIST / 2
         END IF
*
         IF( MYCOL / LEVEL_DIST.LE.( NPCOL-1 ) / LEVEL_DIST-2 ) THEN
            CALL ZGERV2D( ICTXT, BW, BW, AF( ( ODD_SIZE )*BW+1 ), BW, 0,
     $                    COMM_PROC )
*
*         Modify off_diagonal block with diagonal block
*
            CALL ZTRSM( 'L', 'L', 'N', 'N', BW, BW, CONE,
     $                  AF( ( ODD_SIZE+( 1 )*BW )*BW+1 ), BW,
     $                  AF( ( ODD_SIZE )*BW+1 ), BW )
*
*         Calculate contribution from this block to next diagonal block
*
            CALL ZHERK( 'L', 'C', BW, BW, -ONE, AF( ( ODD_SIZE )*BW+1 ),
     $                  BW, ZERO, WORK( 1 ), BW )
*
*         Send contribution to diagonal block's owning processor.
*
            CALL ZGESD2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL+LEVEL_DIST )
*
         END IF
*       End of "if( mycol/level_dist .le. (npcol-1)/level_dist-2 )..."
*
*
*       ****************************************************************
*       Receive off_diagonal block from left and use to finish with this
*         processor.
*
         IF( ( MYCOL / LEVEL_DIST.GT.0 ) .AND.
     $       ( MYCOL / LEVEL_DIST.LE.( NPCOL-1 ) / LEVEL_DIST-1 ) ) THEN
*
            IF( LEVEL_DIST.GT.1 ) THEN
*
*           Receive offdiagonal block from proc level_dist/2 to the left
*
               CALL ZGERV2D( ICTXT, BW, BW,
     $                       AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, 0,
     $                       MYCOL-LEVEL_DIST / 2 )
*
            END IF
*
*         Use diagonal block to modify this offdiagonal block
*
            CALL ZTRSM( 'R', 'L', 'C', 'N', BW, BW, CONE,
     $                  AF( ( ODD_SIZE+1*BW )*BW+1 ), BW,
     $                  AF( ( ODD_SIZE+2*BW )*BW+1 ), BW )
*
*         Use offdiagonal block to calculate modification to diag block
*           of processor to the left
*
            CALL ZHERK( 'L', 'N', BW, BW, -ONE,
     $                  AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, ZERO,
     $                  WORK( 1 ), BW )
*
*         Send contribution to diagonal block's owning processor.
*
            CALL ZGESD2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL-LEVEL_DIST )
*
*         *******************************************************
*
            IF( MYCOL / LEVEL_DIST.LE.( NPCOL-1 ) / LEVEL_DIST-2 ) THEN
*
*           Decide which processor offdiagonal block goes to
*
               IF( ( MOD( MYCOL / ( 2*LEVEL_DIST ), 2 ) ).EQ.0 ) THEN
                  COMM_PROC = MYCOL + LEVEL_DIST
               ELSE
                  COMM_PROC = MYCOL - LEVEL_DIST
               END IF
*
*           Use offdiagonal block and diagonal block to calculate
*             offdiag block to send to neighboring processor. Depending
*             on circumstances, may need to transpose the matrix.
*
               CALL ZGEMM( 'N', 'N', BW, BW, BW, -CONE,
     $                     AF( ( ODD_SIZE+2*BW )*BW+1 ), BW,
     $                     AF( ( ODD_SIZE )*BW+1 ), BW, CZERO,
     $                     WORK( 1 ), BW )
*
*           Send contribution to offdiagonal block's owning processor.
*
               CALL ZGESD2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                       COMM_PROC )
*
            END IF
*
         END IF
*       End of "if( mycol/level_dist.le. (npcol-1)/level_dist -1 )..."
*
   50    CONTINUE
*
*
      ELSE
*
*       CASE UPLO = 'U'
*
********************************************************************
*       PHASE 1: Local computation phase.
********************************************************************
*
*
*       Sizes of the extra triangles communicated bewtween processors
*
         IF( MYCOL.GT.0 ) THEN
            PREV_TRI_SIZE_M = MIN( BW, NUMROC( N, PART_SIZE, MYCOL, 0,
     $                        NPCOL ) )
            PREV_TRI_SIZE_N = MIN( BW, NUMROC( N, PART_SIZE, MYCOL-1, 0,
     $                        NPCOL ) )
         END IF
*
         IF( MYCOL.LT.NPCOL-1 ) THEN
            NEXT_TRI_SIZE_M = MIN( BW, NUMROC( N, PART_SIZE, MYCOL+1, 0,
     $                        NPCOL ) )
            NEXT_TRI_SIZE_N = MIN( BW, NUMROC( N, PART_SIZE, MYCOL, 0,
     $                        NPCOL ) )
         END IF
*
*
*
*       Factor main partition A_i^T = U_i {U_i}^T in each processor
*
         CALL ZPBTRF( UPLO, ODD_SIZE, BW, A( OFST+1 ), LLDA, INFO )
*
         IF( INFO.NE.0 ) THEN
            INFO = INFO + NB*MYCOL
            GO TO 90
         END IF
*
*
         IF( MYCOL.LT.NP-1 ) THEN
*         Apply factorization to odd-even connection block B_i
*
*         Move the connection block in preparation.
*
            CALL ZLACPY( 'L', BW, BW, A( ( OFST+1+ODD_SIZE*LLDA ) ),
     $                   LLDA-1, AF( ( ODD_SIZE+2*BW )*BW+1 ), BW )
*
*         Perform the triangular system solve {L_i}{{B'}_i}^T = {B_i}^T
*
            CALL ZTRTRS( UPLO, 'C', 'N', BW, BW,
     $                   A( OFST+BW+1+( ODD_SIZE-BW )*LLDA ), LLDA-1,
     $                   AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, INFO )
            IF( INFO.NE.0 ) THEN
               INFO = INFO + NB*MYCOL + ODD_SIZE
               GO TO 90
            END IF
*
*         Move the resulting block back to its location in main storage.
*
            CALL ZLACPY( 'L', BW, BW, AF( ( ODD_SIZE+2*BW )*BW+1 ), BW,
     $                   A( ( OFST+1+ODD_SIZE*LLDA ) ), LLDA-1 )
*
*
*         Compute contribution to diagonal block of reduced system.
*          {C'}_i^T = {C_i}^T-{{B'}_i}^T{{B'}_i}
*
*         The following method uses more flops than necessary but
*           does not necessitate the writing of a new BLAS routine.
*
*
            CALL ZHERK( UPLO, 'C', BW, BW, -ONE,
     $                  AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, ONE,
     $                  A( OFST+BW+1+ODD_SIZE*LLDA ), LLDA-1 )
*
         END IF
*       End of "if ( MYCOL .lt. NP-1 )..." loop
*
*
*
         IF( MYCOL.NE.0 ) THEN
*         Discard temporary matrix stored beginning in
*           AF( (odd_size+2*bw)*bw+1 ) and use for
*           off_diagonal block of reduced system.
*
*         Calculate the "spike" fillin, ${L_i} {{G}_i}^T = {D_i}$ .
*
*
*         Copy D block into AF storage for solve.
*
            CALL ZLATCPY( 'L', PREV_TRI_SIZE_N, PREV_TRI_SIZE_M,
     $                    A( OFST+1 ), LLDA-1, AF( 1 ), ODD_SIZE )
*
            CALL ZTBTRS( UPLO, 'C', 'N', ODD_SIZE, BW, BW, A( OFST+1 ),
     $                   LLDA, AF( 1 ), ODD_SIZE, INFO )
            IF( INFO.NE.0 ) THEN
               INFO = INFO + NB*MYCOL + ODD_SIZE
               GO TO 90
            END IF
*
*         Calculate the update block for previous proc, E_i = G_i{G_i}^T
*
            CALL ZHERK( 'L', 'C', BW, ODD_SIZE, -ONE, AF( 1 ), ODD_SIZE,
     $                  ZERO, AF( 1+( ODD_SIZE+2*BW )*BW ), BW )
*
*
*         Initiate send of E_i to previous processor to overlap
*           with next computation.
*
            CALL ZGESD2D( ICTXT, BW, BW, AF( ( ODD_SIZE+2*BW )*BW+1 ),
     $                    BW, 0, MYCOL-1 )
*
*
            IF( MYCOL.LT.NP-1 ) THEN
*
*           Calculate off-diagonal block of reduced system.
*           Note: for ease of use in solution of reduced system, store
*           off-diagonal block in transpose form.
*           {F_i}^T =  {H_i}{{B'}_i}^T
*
*           Copy matrix H_i (the last bw cols of G_i) to AF storage
*             as per requirements of BLAS routine ZTRMM.
*             Since we have G_i^T stored, transpose H_i^T to H_i.
*
               CALL ZLATCPY( 'N', BW, BW, AF( ODD_SIZE-BW+1 ), ODD_SIZE,
     $                       AF( ( ODD_SIZE )*BW+1 ), BW )
*
               CALL ZTRMM( 'R', 'L', 'N', 'N', BW, BW, -CONE,
     $                     A( ( OFST+1+ODD_SIZE*LLDA ) ), LLDA-1,
     $                     AF( ( ODD_SIZE )*BW+1 ), BW )
*
*
            END IF
*
         END IF
*       End of "if ( MYCOL .ne. 0 )..."
*
********************************************************************
*       PHASE 2: Formation and factorization of Reduced System.
********************************************************************
*
*       Gather up local sections of reduced system
*
*     The last processor does not participate in the factorization of
*       the reduced system, having sent its E_i already.
         IF( MYCOL.EQ.NPCOL-1 ) THEN
            GO TO 80
         END IF
*
*       Initiate send of off-diagonal block to overlap with next part.
*       Off-diagonal block needed on neighboring processor to start
*       algorithm.
*
         IF( ( MOD( MYCOL+1, 2 ).EQ.0 ) .AND. ( MYCOL.GT.0 ) ) THEN
*
            CALL ZGESD2D( ICTXT, BW, BW, AF( ( ODD_SIZE )*BW+1 ), BW, 0,
     $                    MYCOL-1 )
*
         END IF
*
*       Transpose last triangular block into AF storage for subsequent
*         operations.
*
         CALL ZLATCPY( 'U', BW, BW, A( OFST+ODD_SIZE*LLDA+1+BW ),
     $                 LLDA-1, AF( ( ODD_SIZE+1*BW )*BW+1 ), BW )
*
*       Receive cont. to diagonal block that is stored on this proc.
*
         IF( MYCOL.LT.NPCOL-1 ) THEN
*
            CALL ZGERV2D( ICTXT, BW, BW,
     $                    AF( ( ODD_SIZE+( 2 )*BW )*BW+1 ), BW, 0,
     $                    MYCOL+1 )
*
*          Add contribution to diagonal block
*
            CALL ZAXPY( BW*BW, CONE, AF( ( ODD_SIZE+( 2 )*BW )*BW+1 ),
     $                  1, AF( ( ODD_SIZE+BW )*BW+1 ), 1 )
*
         END IF
*
*
*       *************************************
*       Modification Loop
*
*       The distance for sending and receiving for each level starts
*         at 1 for the first level.
         LEVEL_DIST = 1
*
*       Do until this proc is needed to modify other procs' equations
*
   60    CONTINUE
         IF( MOD( ( MYCOL+1 ) / LEVEL_DIST, 2 ).NE.0 )
     $      GO TO 70
*
*         Receive and add contribution to diagonal block from the left
*
         IF( MYCOL-LEVEL_DIST.GE.0 ) THEN
            CALL ZGERV2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL-LEVEL_DIST )
*
            CALL ZAXPY( BW*BW, CONE, WORK( 1 ), 1,
     $                  AF( ( ODD_SIZE+BW )*BW+1 ), 1 )
*
         END IF
*
*         Receive and add contribution to diagonal block from the right
*
         IF( MYCOL+LEVEL_DIST.LT.NPCOL-1 ) THEN
            CALL ZGERV2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL+LEVEL_DIST )
*
            CALL ZAXPY( BW*BW, CONE, WORK( 1 ), 1,
     $                  AF( ( ODD_SIZE+BW )*BW+1 ), 1 )
*
         END IF
*
         LEVEL_DIST = LEVEL_DIST*2
*
         GO TO 60
   70    CONTINUE
*       [End of GOTO Loop]
*
*
*       *********************************
*       Calculate and use this proc's blocks to modify other procs'...
*
*       Factor diagonal block
*
         CALL ZPOTRF( 'L', BW, AF( ( ODD_SIZE+BW )*BW+1 ), BW, INFO )
*
         IF( INFO.NE.0 ) THEN
            INFO = INFO + N - ( NPCOL-1 )*BW + MYCOL*BW
            GO TO 90
         END IF
*
*       ****************************************************************
*       Receive offdiagonal block from processor to right.
*         If this is the first group of processors, the receive comes
*         from a different processor than otherwise.
*
         IF( LEVEL_DIST.EQ.1 ) THEN
            COMM_PROC = MYCOL + 1
*
*           Move block into place that it will be expected to be for
*             calcs.
*
            CALL ZLACPY( 'N', BW, BW, AF( ( ODD_SIZE )*BW+1 ), BW,
     $                   AF( ( ODD_SIZE+2*BW )*BW+1 ), BW )
*
         ELSE
            COMM_PROC = MYCOL + LEVEL_DIST / 2
         END IF
*
         IF( MYCOL / LEVEL_DIST.LE.( NPCOL-1 ) / LEVEL_DIST-2 ) THEN
            CALL ZGERV2D( ICTXT, BW, BW, AF( ( ODD_SIZE )*BW+1 ), BW, 0,
     $                    COMM_PROC )
*
*         Modify off_diagonal block with diagonal block
*
            CALL ZTRSM( 'L', 'L', 'N', 'N', BW, BW, CONE,
     $                  AF( ( ODD_SIZE+( 1 )*BW )*BW+1 ), BW,
     $                  AF( ( ODD_SIZE )*BW+1 ), BW )
*
*         Calculate contribution from this block to next diagonal block
*
            CALL ZHERK( 'L', 'C', BW, BW, -ONE, AF( ( ODD_SIZE )*BW+1 ),
     $                  BW, ZERO, WORK( 1 ), BW )
*
*         Send contribution to diagonal block's owning processor.
*
            CALL ZGESD2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL+LEVEL_DIST )
*
         END IF
*       End of "if( mycol/level_dist .le. (npcol-1)/level_dist-2 )..."
*
*
*       ****************************************************************
*       Receive off_diagonal block from left and use to finish with this
*         processor.
*
         IF( ( MYCOL / LEVEL_DIST.GT.0 ) .AND.
     $       ( MYCOL / LEVEL_DIST.LE.( NPCOL-1 ) / LEVEL_DIST-1 ) ) THEN
*
            IF( LEVEL_DIST.GT.1 ) THEN
*
*           Receive offdiagonal block from proc level_dist/2 to the left
*
               CALL ZGERV2D( ICTXT, BW, BW,
     $                       AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, 0,
     $                       MYCOL-LEVEL_DIST / 2 )
*
            END IF
*
*         Use diagonal block to modify this offdiagonal block
*
            CALL ZTRSM( 'R', 'L', 'C', 'N', BW, BW, CONE,
     $                  AF( ( ODD_SIZE+1*BW )*BW+1 ), BW,
     $                  AF( ( ODD_SIZE+2*BW )*BW+1 ), BW )
*
*         Use offdiagonal block to calculate modification to diag block
*           of processor to the left
*
            CALL ZHERK( 'L', 'N', BW, BW, -ONE,
     $                  AF( ( ODD_SIZE+2*BW )*BW+1 ), BW, ZERO,
     $                  WORK( 1 ), BW )
*
*         Send contribution to diagonal block's owning processor.
*
            CALL ZGESD2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                    MYCOL-LEVEL_DIST )
*
*         *******************************************************
*
            IF( MYCOL / LEVEL_DIST.LE.( NPCOL-1 ) / LEVEL_DIST-2 ) THEN
*
*           Decide which processor offdiagonal block goes to
*
               IF( ( MOD( MYCOL / ( 2*LEVEL_DIST ), 2 ) ).EQ.0 ) THEN
                  COMM_PROC = MYCOL + LEVEL_DIST
               ELSE
                  COMM_PROC = MYCOL - LEVEL_DIST
               END IF
*
*           Use offdiagonal block and diagonal block to calculate
*             offdiag block to send to neighboring processor. Depending
*             on circumstances, may need to transpose the matrix.
*
               CALL ZGEMM( 'N', 'N', BW, BW, BW, -CONE,
     $                     AF( ( ODD_SIZE+2*BW )*BW+1 ), BW,
     $                     AF( ( ODD_SIZE )*BW+1 ), BW, CZERO,
     $                     WORK( 1 ), BW )
*
*           Send contribution to offdiagonal block's owning processor.
*
               CALL ZGESD2D( ICTXT, BW, BW, WORK( 1 ), BW, 0,
     $                       COMM_PROC )
*
            END IF
*
         END IF
*       End of "if( mycol/level_dist.le. (npcol-1)/level_dist -1 )..."
*
   80    CONTINUE
*
*
      END IF
*
   90 CONTINUE
*
*
*     Free BLACS space used to hold standard-form grid.
*
      ICTXT = ICTXT_SAVE
      IF( ICTXT.NE.ICTXT_NEW ) THEN
         CALL BLACS_GRIDEXIT( ICTXT_NEW )
      END IF
*
  100 CONTINUE
*
*     Restore saved input parameters
*
      NP = NP_SAVE
*
      RETURN
*
*     End of PZPBTRF
*
      END
