      SUBROUTINE PZPBTRS( UPLO, N, BW, NRHS, A, JA, DESCA, B, IB, DESCB,
     $                    AF, LAF, WORK, LWORK, INFO )
*
*  -- ScaLAPACK routine (version 1.2 ALPHA) --
*     University of Tennessee, Knoxville, Oak Ridge National Laboratory,
*     and University of California, Berkeley.
*     May 10, 1996
*
*     .. Scalar Arguments ..
      CHARACTER          UPLO
      INTEGER            BW, IB, INFO, JA, LAF, LWORK, N, NRHS
*     ..
*     .. Array Arguments ..
      INTEGER            DESCA( * ), DESCB( * )
      COMPLEX*16         A( * ), AF( * ), B( * ), WORK( * )
*     ..
*
*  Purpose
*  =======
*
*  PZPBTRS solves a system of linear equations
*
*                      sub( A ) * X = sub( B )
*
*  where sub( A ) is the matrix used to produce the Cholesky factors
*  stored in A(1:N,JA:JA+N-1). sub( A ) is an N-by-N banded
*  symmetric positive definite distributed matrix and its Cholesky
*  factor is also N-by-N and banded.
*  Depending on the value of UPLO, A stores either U or L in the equn
*  sub( A ) = U**T*U or L*L**T as computed by PZPBTRF.
*
*  sub( B ) denotes the distributed matrix B(IB:IB+N-1,1:NRHS).
*
*  Routine PZPBTRF MUST be called first.
*
*  =====================================================================
*
*  Arguments
*  =========
*
*  UPLO    (global input) CHARACTER
*          = 'U':  Upper triangle of sub( A ) is stored;
*          = 'L':  Lower triangle of sub( A ) is stored.
*
*  N       (global input) INTEGER
*          The number of rows and columns to be operated on, i.e. the
*          order of the distributed submatrix sub( A ). N >= 0.
*
*  BW      Number of subdiagonals
*
*
*  A       (local input/local output) COMPLEX*16 pointer into the
*          local memory to an array of dimension with first dimension
*          LLD_A.
*          On entry, this array contains the local pieces of the
*          This local portion is stored in the packed banded format
*            used in LAPACK.
*
*  JA      (global input) INTEGER
*          A's global column index, which points to the beginning of
*          the submatrix which is to be operated on.
*
*  DESCA   (global and local input) INTEGER array of dimension >=7
*          if 1D type, >=9 if 2D type.
*          The array descriptor for the distributed matrix A.
*
*  INFO    (global output) INTEGER
*          = 0:  successful exit
*          < 0:  If the i-th argument is an array and the j-entry had
*                an illegal value, then INFO = -(i*100+j), if the i-th
*                argument is a scalar and had an illegal value, then
*                INFO = -i.
*          > 0:  If INFO = K, the leading minor of order K,
*                A(IA:IA+K-1,JA:JA+K-1) is not positive definite, and
*                the factorization could not be completed.
*
*  =====================================================================
*
*
*  Restrictions
*  ============
*
*  The following are restrictions on the input parameters. Many of these
*    are temporary and will be removed in future releases, while others
*    may reflect fundamental technical limitations.
*
*    Non-cyclic restriction: VERY IMPORTANT!
*      P*NB<= JA+N.
*      The mapping for matrices must be blocked, reflecting the nature
*      of the divide and conquer algorithm as a task-parallel algorithm.
*
*    Blocksize cannot be too small:
*      NB >= 2*BW
*      The bulk of parallel computation is done on the matrix of size
*      NB-BW on each processor. If this is too small, divide and conquer
*      is a poor choice of algorithm.
*
*    Submatrix reference:
*      JA = IB
*      Alignment restriction that prevents unnecessary communication.
*
*
*  =====================================================================
*
*
*  Notes
*  =====
*
*  If the factorization routine and the solve routine are to be called
*    separately (to solve various sets of righthand sides using the same
*    coefficient matrix), the auxiliary space AF *must not be altered*
*    between calls to the factorization routine and the solve routine.
*
*  The best algorithm for solving banded and tridiagonal linear systems
*    depends on a variety of parameters, especially the bandwidth.
*    Currently, only algorithms designed for the case N/P >> bw are
*    implemented. These go by many names, including Divide and Conquer,
*    Partitioning, domain decomposition-type, etc.
*
*  Algorithm description: Divide and Conquer
*
*    The Divide and Conqer algorithm assumes the matrix is narrowly
*      banded compared with the number of equations. In this situation,
*      it is best to store the matrix one-dimensionally.
*      The basic algorithm divides the banded matrix up into a small
*      number of pieces (usually P with one stored on each processor)
*      and then proceeds in 2 phases for the factorization or 3 for the
*      solution of a linear system.
*      1) Local Phase:
*         The individual pieces are factored independently and in
*         parallel. These factors are applied to the matrix creating
*         fillin. For a linear system, a local front solve is
*         performed in this stage.
*      2) Reduced System Phase:
*         A small ( BW* (P-1) ) matrix/system is formed representing the
*         interaction of the larger blocks. It can be factored in
*         several ways. Currently a parallel Block Cyclic Reduction
*         algorithm is used. For a linear system, a parallel front solve
*         followed by an analagous backsolve, both using the structure
*         of the factored matrix, are performed.
*      3) Backsubsitution Phase:
*         For a linear system, a local backsubstitution is performed on
*         each processor in parallel.
*
*  Descriptors now have *types* and differ from ScaLAPACK 1.0.
*
*  Banded codes can use either the standard two-dimensional descriptors
*    or new one-dimensional descriptors, though the processor grid in
*    both cases *must be one-dimensional*. We describe both types below.
*
*  A description vector is associated with each 2D block-cyclicly dis-
*  tributed matrix.  This vector stores the information required to
*  establish the mapping between a matrix entry and its corresponding
*  process and memory location.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN      EXPLANATION
*  --------------- -------------- --------------------------------------
*  DT_A   (global) DESCA( DT_ )   The descriptor type.
*  CTXT_A (global) DESCA( CTXT_ ) The BLACS context handle, indicating
*                                 the BLACS process grid A is distribu-
*                                 ted over. The context itself is glo-
*                                 bal, but the handle (the integer
*                                 value) may vary.
*  M_A    (global) DESCA( M_ )    The number of rows in the distributed
*                                 matrix A.
*  N_A    (global) DESCA( N_ )    The number of columns in the distri-
*                                 buted matrix A.
*  MB_A   (global) DESCA( MB_ )   The blocking factor used to distribute
*                                 the rows of A.
*  NB_A   (global) DESCA( NB_ )   The blocking factor used to distribute
*                                 the columns of A.
*  RSRC_A (global) DESCA( RSRC_ ) The process row over which the first
*                                 row of the matrix A is distributed.
*  CSRC_A (global) DESCA( CSRC_ ) The process column over which the
*                                 first column of A is distributed.
*  LLD_A  (local)  DESCA( LLD_ )  The leading dimension of the local
*                                 array storing the local blocks of the
*                                 distributed matrix A.
*                                 LLD_A >= MAX(1,LOCp(M_A)).
*
*  Let K be the number of rows or columns of a distributed matrix,
*  and assume that its process grid has dimension p x q.
*  LOCp( K ) denotes the number of elements of K that a process
*  would receive if K were distributed over the p processes of its
*  process column.
*  Similarly, LOCq( K ) denotes the number of elements of K that a
*  process would receive if K were distributed over the q processes of
*  its process row.
*  The values of LOCp() and LOCq() may be determined via a call to the
*  ScaLAPACK tool function, NUMROC:
*          LOCp( M ) = NUMROC( M, MB_A, MYROW, RSRC_A, NPROW ),
*          LOCq( N ) = NUMROC( N, NB_A, MYCOL, CSRC_A, NPCOL ).
*
*
*  One-dimensional descriptors:
*
*  One-dimensional descriptors are a new addition to ScaLAPACK since
*    version 1.0. They simplify and shorten the descriptor for 1D
*    matrices. Such a descriptor is associated with each one-
*    dimensionally distributed matrix in ScaLAPACK. This descriptor
*    allows ScaLAPACK programs to be addressed in global indices by
*    providing the means to translate between global indices and local
*    indices in a way that insulates the user from such details.
*
*  Since ScaLAPACK supports two-dimensional matrices as the fundamental
*    object, we allow for matrices to be distributed either over the
*    first dimension (as if the grid were P-by-1) or the second
*    dimension (as if the grid were 1-by-P). This choice is made in the
*    first entry of the descriptor as described below.
*    IMPORTANT NOTE: the actual BLACS grid represented by the
*    CTXT entry in the descriptor may be *either*  P-by-1 or 1-by-P
*    for either one-dimensional descriptor type. The ScaLAPACK
*    routines will interpret these properly depending on the TYPE.
*    ScaLAPACK routines *do not support intercontext operations* so that
*    the grid passed to a single ScaLAPACK routine *must be the same*
*    for all matrix descriptors passed to that routine.
*
*  In the following comments, the character _ should be read as
*  "of the distributed matrix".  Let A be a generic term for any 2D
*  block cyclicly distributed matrix.  Its description vector is DESCA:
*
*  NOTATION        STORED IN  EXPLANATION
*  --------------- ---------- ------------------------------------------
*  TYPE_A (global) DESCA( 1 ) The orientation of the one-dimensional
*                             grid.
*                             TYPE_A = 502: P-by-1 grid.
*                             TYPE_A = 501: 1-by-P grid.
*  CTXT_A (global) DESCA( 2 ) The BLACS context handle, indicating the
*                             BLACS process grid A is distributed over.
*                             The context itself is global, but the
*                             handle (the integer value) may vary.
*  N_A    (global) DESCA( 3 ) The size of the matrix dimension being
*                             distributed.
*  NB_A   (global) DESCA( 4 ) The blocking factor used to distribute
*                             the distributed dimension of the matrix.
*  SRC_A  (global) DESCA( 5 ) The process row or column over which the
*                             1st element of the matrix is distributed.
*  LLD_A  (local)  DESCA( 6 ) The leading dimension of the local array
*                             storing the local blocks of the distri-
*                             buted matrix A. Minimum value of LLD_A
*                             depends on TYPE_A.
*  Reserved        DESCA( 7 ) Reserved for future use.
*
*
*  =====================================================================
*
*  Implemented by: Andrew J. Cleary, May 10, 1996
*
*  =====================================================================
*
*     .. Parameters ..
      INTEGER            DESCMULT, BIGNUM
      PARAMETER          ( DESCMULT = 100, BIGNUM = DESCMULT*DESCMULT )
      INTEGER            BLOCK_CYCLIC_2D, CSRC_, CTXT_, DLEN_, DT_,
     $                   LLD_, MB_, M_, NB_, N_, RSRC_
      PARAMETER          ( BLOCK_CYCLIC_2D = 1, DLEN_ = 9, DT_ = 1,
     $                     CTXT_ = 2, M_ = 3, N_ = 4, MB_ = 5, NB_ = 6,
     $                     RSRC_ = 7, CSRC_ = 8, LLD_ = 9 )
*     ..
*     .. Local Scalars ..
      INTEGER            CSRC, ICTXT, IDUM1, LLDA, LLDB, MYCOL, MYROW,
     $                   NB, NP, NPCOL, NPROW, RETURN_CODE, STORE_N,
     $                   WORK_SIZE_MIN
*     ..
*     .. Local Arrays ..
      INTEGER            DESCA_1XP( 7 ), DESCB_PX1( 7 ),
     $                   PARAM_CHECK( 16, 3 )
*     ..
*     .. External Subroutines ..
      EXTERNAL           BLACS_GRIDINFO, DESC_CONVERT, GLOBCHK, PXERBLA,
     $                   PZTBTRS
*     ..
*     .. External Functions ..
      LOGICAL            LSAME
      EXTERNAL           LSAME
*     ..
*     .. Intrinsic Functions ..
      INTRINSIC          ICHAR, MOD
*     ..
*     .. Executable Statements ..
*
*     Quick return if possible
*
      IF( N.EQ.0 )
     $   RETURN
*
*     Test the input parameters
*
      INFO = 0
*
*
*     Convert descriptor into standard form for easy access to
*        parameters, check that grid is of right shape.
*
      DESCA_1XP( 1 ) = 501
      DESCB_PX1( 1 ) = 502
*
      CALL DESC_CONVERT( DESCA, DESCA_1XP, RETURN_CODE )
*
      IF( RETURN_CODE.NE.0 ) THEN
         INFO = -( 7*100+2 )
      END IF
*
*
      CALL DESC_CONVERT( DESCB, DESCB_PX1, RETURN_CODE )
*
      IF( RETURN_CODE.NE.0 ) THEN
         INFO = -( 10*100+2 )
      END IF
*
*     Consistency checks for DESCA and DESCB.
*
*     Context must be the same
      IF( DESCA_1XP( 2 ).NE.DESCB_PX1( 2 ) ) THEN
         INFO = -( 10*100+2 )
      END IF
*
*        These are alignment restrictions that may or may not be removed
*        in future releases. -Andy Cleary, April 14, 1996.
*
*     Block sizes must be the same
      IF( DESCA_1XP( 4 ).NE.DESCB_PX1( 4 ) ) THEN
         INFO = -( 10*100+4 )
      END IF
*
*     Source processor must be the same
      IF( DESCA_1XP( 5 ).NE.DESCB_PX1( 5 ) ) THEN
         INFO = -( 10*100+5 )
      END IF
*
*     Get values out of descriptor for use in code.
*
      ICTXT = DESCA_1XP( 2 )
      CSRC = DESCA_1XP( 5 )
      NB = DESCA_1XP( 4 )
      LLDA = DESCA_1XP( 6 )
      STORE_N = DESCA_1XP( 3 )
      LLDB = DESCB_PX1( 6 )
*
*     Get grid parameters
*
      CALL BLACS_GRIDINFO( ICTXT, NPROW, NPCOL, MYROW, MYCOL )
      NP = NPROW*NPCOL
*
*
      IF( LSAME( UPLO, 'U' ) ) THEN
         IDUM1 = ICHAR( 'U' )
      ELSE IF( LSAME( UPLO, 'L' ) ) THEN
         IDUM1 = ICHAR( 'L' )
      ELSE
         INFO = -1
      END IF
*
      IF( N.LT.0 ) THEN
         INFO = -2
      END IF
*
      IF( N+JA-1.LT.STORE_N ) THEN
         INFO = -( 7*100+3 )
      END IF
*
      IF( ( BW.GT.N-1 ) .AND. ( BW.LT.0 ) ) THEN
         INFO = -4
      END IF
*
      IF( LLDA.LT.BW+1 ) THEN
         INFO = -( 7*100+6 )
      END IF
*
      IF( LLDB.LT.NB ) THEN
         INFO = -( 10*100+6 )
      END IF
*
      IF( NRHS.LT.1 ) THEN
         INFO = -3
      END IF
*
*     Argument checking that is specific to Divide & Conquer routine
*
      IF( NPROW.NE.1 ) THEN
         INFO = -( 7*100+2 )
      END IF
*
      IF( N.GT.NP*NB-MOD( JA-1, NB ) ) THEN
         INFO = -( 2 )
         CALL PXERBLA( ICTXT, 'PZPBTRS, D&C alg.: only 1 block per proc'
     $                 , INFO )
         RETURN
      END IF
*
*     Current restrictions on Divide and Conquer that may be lifted
*
      IF( CSRC.NE.0 ) THEN
         INFO = -( 7*100+5 )
      END IF
*
      IF( JA.NE.1 ) THEN
         INFO = -( 6 )
      END IF
*
*
      WORK_SIZE_MIN = BW*NRHS
      IF( LWORK.LT.WORK_SIZE_MIN ) THEN
         INFO = -14
*        put minimum value of work into work( 1 )
         WORK( 1 ) = WORK_SIZE_MIN
         CALL PXERBLA( ICTXT, 'PZPBTRS1Dd_and_c: worksize error',
     $                 -INFO )
         RETURN
      END IF
*
*     Pack params and positions into arrays for global consistency check
*
      PARAM_CHECK( 16, 1 ) = DESCB( 5 )
      PARAM_CHECK( 15, 1 ) = DESCB( 4 )
      PARAM_CHECK( 14, 1 ) = DESCB( 3 )
      PARAM_CHECK( 13, 1 ) = DESCB( 2 )
      PARAM_CHECK( 12, 1 ) = DESCB( 1 )
      PARAM_CHECK( 11, 1 ) = IB
      PARAM_CHECK( 10, 1 ) = DESCA( 5 )
      PARAM_CHECK( 9, 1 ) = DESCA( 4 )
      PARAM_CHECK( 8, 1 ) = DESCA( 3 )
      PARAM_CHECK( 7, 1 ) = DESCA( 2 )
      PARAM_CHECK( 6, 1 ) = DESCA( 1 )
      PARAM_CHECK( 5, 1 ) = JA
      PARAM_CHECK( 4, 1 ) = NRHS
      PARAM_CHECK( 3, 1 ) = BW
      PARAM_CHECK( 2, 1 ) = N
      PARAM_CHECK( 1, 1 ) = IDUM1
*
      PARAM_CHECK( 16, 2 ) = 1005
      PARAM_CHECK( 15, 2 ) = 1004
      PARAM_CHECK( 14, 2 ) = 1003
      PARAM_CHECK( 13, 2 ) = 1002
      PARAM_CHECK( 12, 2 ) = 1001
      PARAM_CHECK( 11, 2 ) = 9
      PARAM_CHECK( 10, 2 ) = 705
      PARAM_CHECK( 9, 2 ) = 704
      PARAM_CHECK( 8, 2 ) = 703
      PARAM_CHECK( 7, 2 ) = 702
      PARAM_CHECK( 6, 2 ) = 701
      PARAM_CHECK( 5, 2 ) = 6
      PARAM_CHECK( 4, 2 ) = 4
      PARAM_CHECK( 3, 2 ) = 3
      PARAM_CHECK( 2, 2 ) = 2
      PARAM_CHECK( 1, 2 ) = 1
*
*     Want to find errors with MIN( ), so if no error, set it to a big
*     number. If there already is an error, multiply by the the
*     descriptor multiplier.
*
      IF( INFO.GE.0 ) THEN
         INFO = BIGNUM
      ELSE IF( INFO.LT.-DESCMULT ) THEN
         INFO = -INFO
      ELSE
         INFO = -INFO*DESCMULT
      END IF
*
*     Check consistency across processors
*
      CALL GLOBCHK( ICTXT, 16, PARAM_CHECK, 16, PARAM_CHECK( 1, 3 ),
     $              INFO )
*
*     Prepare output: set info = 0 if no error, and divide by DESCMULT
*     if error is not in a descriptor entry.
*
      IF( INFO.EQ.BIGNUM ) THEN
         INFO = 0
      ELSE IF( MOD( INFO, DESCMULT ).EQ.0 ) THEN
         INFO = -INFO / DESCMULT
      ELSE
         INFO = -INFO
      END IF
*
      IF( INFO.LT.0 ) THEN
         CALL PXERBLA( ICTXT, 'PZPBTRS', -INFO )
         RETURN
      END IF
*
*
*
*     Begin main code
*
      INFO = 0
*
*     Call frontsolve routine
*
      CALL PZTBTRS( UPLO, 'N', N, BW, NRHS, A, JA, DESCA, B, IB, DESCB,
     $              AF, LAF, WORK, LWORK, INFO )
*
*     Check info for error conditions
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICTXT, 'PZPBTRS', -INFO )
         RETURN
      END IF
*
*     Call backsolve routine
*
      CALL PZTBTRS( UPLO, 'C', N, BW, NRHS, A, JA, DESCA, B, IB, DESCB,
     $              AF, LAF, WORK, LWORK, INFO )
*
*     Check info for error conditions
*
      IF( INFO.NE.0 ) THEN
         CALL PXERBLA( ICTXT, 'PZPBTRS', -INFO )
         RETURN
      END IF
   10 CONTINUE
*
*
*
      RETURN
*
*     End of PZPBTRS
*
      END
